SystemOrganization addCategory: #ConfigurationOfNBSQLite3!

Object subclass: #ConfigurationOfNBSQLite3
	instanceVariableNames: 'project'
	classVariableNames: 'LastVersionLoad'
	poolDictionaries: ''
	category: 'ConfigurationOfNBSQLite3'!

!ConfigurationOfNBSQLite3 commentStamp: 'TorstenBergmann 3/1/2015 21:45' prior: 0!
# NBSQLite3 for Pharo

## Project info

The "NBSQLite3 for Pharo" is a project to provide an API and access to the [SQLite3](http://www.sqlite.org/) database from within the Pharo image.

With this tool you can use the small relational SQL engine into own Pharo projects.

### Project location
The project is located on STHub at
[http://smalltalkhub.com/#!!/~PharoExtras/NBSQLite3/settings](http://smalltalkhub.com/#!!/~PharoExtras/NBSQLite3/settings)

### History
SQLite is a small SQL database engine that you can use if you are in need of a leightweight RDBMS solution for you application. There a various bindings to it in all the different programming languages.

For Smalltalk initially there was a project called "SQLite" with a binding to SQLite3 database engine. The native calls were implemented using the foreign function interface project "FFI". This work initially started in Squeak and was later ported over to Pharo and is now hosted on [STHub in project "SQLite"](http://smalltalkhub.com/#!!/~TorstenBergmann/SQLite).

Afterwards, in March 2014 [a blog post about wrapping the SQLite3 API using NativeBoost](http://www.samadhiweb.com/blog/2014.03.01.nbsqlite3.html) appeared on the Samadhiweb blog. 

The author "Pierce" has provided the initial implementation as a package on [SqueakSource3](http://ss3.gemstone.com/ss/NBSQLite3.html/Wiki). Aftwards it was cleaned up and refactored a little bit and is now provided as a project  ["NBSQLite3" on SmalltalkHub](http://smalltalkhub.com/#!!/~PharoExtras/NBSQLite3/settings).

### License
NBSQLite3 is licensed under MIT License.

## Installation

You can access NBSQLite3 from the Pharo configuration browser. Just select *"Tools"* -> *"Configuration Browser"* from the world menu, enter "NBSQLite3" and install the stable version. 

Alternatively to install the packages manually into your Pharo image just evaluate:

    Gofer new
        url: 'http://smalltalkhub.com/mc/PharoExtras/NBSQLite3/main';
        package: 'ConfigurationOfNBSQLite3';
        load.
    ((Smalltalk at: #ConfigurationOfNBSQLite3) project stableVersion) load.

If you need access to the SQLite3 database engine you need at a minimum the "NBSQLite3-Core" package which is loaded by the ConfigurationOfNBSQLite3 class. 

This package includes all necessary  API to access the sqlite engine. 

## Use in your own application

### Providing the library files
To use the sqlite3 database engine in own Pharo projects you need to install the "sqlite3" database engine first. Basically it is only a shared library component that has to be found by the Pharo virtual machine.
***Note that you have to use the 32 bit version even when running on a 64 bit machine.***

#### Library files on Windows
For instance on Windows operating system you can download the "sqlite3.dll" component from [http://www.sqlite.org/](http://www.sqlite.org/) and copy it into the same directory where the Pharo VM (Pharo.exe) is. 
#### Library files on CentOS (Linux)
On CentOS linux you can use

	sudo yum install sqlite.i686

to load sqlite3 in CentOS linux. Then copy the lib from /usr/lib into the pharo-vm/ folder and create a link:

	ln -s libsqlite3.so.0.8.6 libsqlite3

The 'sqlite3' name is defined in SqliteLibrary>>moduleName.

#### Troubleshooting and library files on other platforms
Basically install libsqlite3.so so that it is accessible to the Pharo virtual machine and make sure you used the 32 bit version. 
	
#### Check installation
After providing the library and loading NBSQLite3 as described before you can run the unit tests to see if anything is correctly setup and the virtual machine is able to find the sqlite3 engine.

### Use in your code
#### Creating a connection
The most important class to use for creating a database is the class ***NBSQLite3Connection***. For instance the following expression:

	NBSQLite3Connection on: 'myfile.db'.
	
will setup a new connection with the sqlite engine using the database file myfile.db.
	
#### Open a connection
To open a connection just send the message #open to the connection:

	(NBSQLite3Connection on: 'myfile.db') open.

#### Execute a statement
To execute a statement just use the #execute: selector:

	|db|
	db := NBSQLite3Connection on: 'myfile.db'.
	db open.
	db execute: 'CREATE TABLE IF NOT EXISTS BLOGPOSTS (id integer primary key, post text);'.

Lets also add some data by inserting a new row:

		db execute: 'INSERT INTO BLOGPOSTS values (1, "HelloWorld");'.

And query the database again:

	db execute: 'SELECT * FROM BLOGPOSTS;'.

#### Working with prepared statements
With the #prepare: message you are able to retrieve a prepared stament: 

    s := db prepare: 'select * from BLOGPOSTS'.
	s basicExecute: [  :row |
		row dataValuesAvailable inspect ].
	s finalize.
	
#### Working with transactions
If you need transactions use the messages #beginTransaction and #rollbackTransaction or #commitTransaction.

	res := db beginTransaction.
	
	rs := db execute: '...'.
	rs close.

	res := db commitTransaction.
	

#### Other
Checkout the test suite and the examples in the package ***NBSQLite3-Examples***.
	
# Packages

- *NBSQLite3-Core* - package with the core, contains anything you need in an own app
- *NBSQLite3-Tests-Core* - package with the SUnit tests for the core package
- *NBSQLite3-Examples* - package with examples
- *NBSQLite3-Glorp* - support for [GLORP](http://www.glorp.org)
- *NBSQLite3-Tests-Glorp* - tests for GLORP

# Testing
The package comes with many tests in the package ***NBSQLite3-Tests-Core*** and ***NBSQLite3-Tests-Glorp***. Just use the SUnit TestRunner to run them.

# CI Jobs
The project is controlled by a CI server under:  [https://ci.inria.fr/dbxtalk/view/SQLite/job/NBSQLite3-NativeDriver-SQLite](https://ci.inria.fr/dbxtalk/view/SQLite/job/NBSQLite3-NativeDriver-SQLite)
!

!ConfigurationOfNBSQLite3 class methodsFor: 'development support' stamp: 'TorstenBergmann 12/15/2014 12:16'!
DevelopmentSupport

"See the methods in the 'development support' category on the class-side of MetacelloBaseConfiguration. Decide what development support methods you would like to use and copy them the the class-side of your configuration."
	<apiDocumentation>
! !

!ConfigurationOfNBSQLite3 class methodsFor: 'private' stamp: 'TorstenBergmann 12/15/2014 12:16'!
baseConfigurationClassIfAbsent: aBlock

	^Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			self ensureMetacelloBaseConfiguration.
			Smalltalk at: #MetacelloBaseConfiguration ifAbsent: aBlock ].
! !

!ConfigurationOfNBSQLite3 class methodsFor: 'catalog' stamp: 'PierceNg 1/6/2016 21:48'!
catalogContactInfo
	^ 'pharo-users mailing list'! !

!ConfigurationOfNBSQLite3 class methodsFor: 'catalog' stamp: 'PierceNg 1/6/2016 21:49'!
catalogDescription
	^ 'NBSQLite3 is a NativeBoost interface to SQLite.'! !

!ConfigurationOfNBSQLite3 class methodsFor: 'catalog' stamp: 'PierceNg 1/6/2016 21:47'!
catalogKeywords
	^ #('Database' 'Persistence' 'SQL' 'SQLite' 'NativeBoost') ! !

!ConfigurationOfNBSQLite3 class methodsFor: 'private' stamp: 'TorstenBergmann 12/15/2014 12:16'!
ensureMetacello

	(self baseConfigurationClassIfAbsent: []) ensureMetacello! !

!ConfigurationOfNBSQLite3 class methodsFor: 'private' stamp: 'TorstenBergmann 12/15/2014 12:16'!
ensureMetacelloBaseConfiguration

	Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			| repository version |
			repository := MCHttpRepository location: 'http://seaside.gemstone.com/ss/metacello' user: '' password: ''.
			repository
				versionReaderForFileNamed: 'Metacello-Base-DaleHenrichs.2.mcz'
				do: [ :reader | 
					version := reader version.
					version load.
					version workingCopy repositoryGroup addRepository: repository ] ]! !

!ConfigurationOfNBSQLite3 class methodsFor: 'metacello tool support' stamp: 'TorstenBergmann 12/15/2014 12:16'!
isMetacelloConfig
	"Answer true and the Metacello tools will operate on you"
	
	^true! !

!ConfigurationOfNBSQLite3 class methodsFor: 'loading' stamp: 'TorstenBergmann 12/15/2014 12:16'!
load
	"Load the #stable version defined for this platform. The #stable version is the version that is recommended to be used on this platform."

	"self load"

	<apiDocumentation>
	^(self project version: #stable) load! !

!ConfigurationOfNBSQLite3 class methodsFor: 'loading' stamp: 'TorstenBergmann 12/15/2014 12:16'!
loadBleedingEdge
	"Load the latest versions of the mcz files defined for this project. It is not likely that the #bleedingEdge has been tested."

	"self loadBleedingEdge"

	<apiDocumentation>
	^(self project version: #bleedingEdge) load! !

!ConfigurationOfNBSQLite3 class methodsFor: 'loading' stamp: 'TorstenBergmann 12/15/2014 12:16'!
loadDevelopment
	"Load the #development version defined for this platform. The #development version will change over time and is not expected to be stable."

	"self loadDevelopment"

	<apiDocumentation>
	^(self project version: #development) load! !

!ConfigurationOfNBSQLite3 class methodsFor: 'accessing' stamp: 'TorstenBergmann 12/15/2014 12:16'!
project

	^self new project! !

!ConfigurationOfNBSQLite3 class methodsFor: 'development support' stamp: 'TorstenBergmann 12/15/2014 12:16'!
validate
	"Check the configuration for Errors, Critical Warnings, and Warnings (see class comment for MetacelloMCVersionValidator for more information). 
	Errors identify specification issues that will result in unexpected behaviour when you load the configuration. 
	Critical Warnings identify specification issues that may result in unexpected behavior when you load the configuration.
	Warnings identify specification issues that are technically correct, but are worth take a look at."

	"self validate"

	<apiDocumentation>
	self ensureMetacello.
	^ ((Smalltalk at: #MetacelloToolBox) validateConfiguration: self debug: #() recurse: false) explore! !

!ConfigurationOfNBSQLite3 methodsFor: 'baselines' stamp: 'TorstenBergmann 12/15/2014 12:27'!
baseline01: spec
	<version: '0.1-baseline'>

	spec for: #'common' do: [
		spec blessing: #'baseline'.
		spec repository: 'http://smalltalkhub.com/mc/PharoExtras/NBSQLite3/main/'.
		spec 
			package: #'NBSQLite3-Core';
			package: #'NBSQLite3-Examples' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Glorp' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Tests-Core' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Tests-Glorp' with: [
				spec requires: #(#'NBSQLite3-Glorp' #'NBSQLite3-Tests-Core' ). ].
		spec 
			group: 'Core' with: #(#'NBSQLite3-Core' );
			group: 'default' with: #('Core' 'Tests');
			group: 'Examples' with: #(#'NBSQLite3-Examples' );
			group: 'Glorp' with: #(#'NBSQLite3-Glorp' );
			group: 'Tests' with: #(#'NBSQLite3-Tests-Core' );
			group: 'Tests-Glorp' with: #(#'NBSQLite3-Tests-Glorp' ). ].
! !

!ConfigurationOfNBSQLite3 methodsFor: 'baselines' stamp: 'TorstenBergmann 9/27/2015 11:02'!
baseline12: spec
	<version: '1.2-baseline'>

	spec for: #'common' do: [
		spec blessing: #'baseline'.
		spec repository: 'http://smalltalkhub.com/mc/PharoExtras/NBSQLite3/main/'.
		spec 
			package: #'NBSQLite3-Core';
			package: #'NBSQLite3-Examples' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Glorp' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Tests-Core' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Tests-Glorp' with: [
				spec requires: #(#'NBSQLite3-Glorp' #'NBSQLite3-Tests-Core' ). ].
		spec 
			group: 'Core' with: #(#'NBSQLite3-Core' );
			group: 'default' with: #('Core' 'Tests');
			group: 'Examples' with: #(#'NBSQLite3-Examples' );
			group: 'Glorp' with: #(#'NBSQLite3-Glorp' );
			group: 'Tests' with: #(#'NBSQLite3-Tests-Core' );
			group: 'Tests-Glorp' with: #(#'NBSQLite3-Tests-Glorp' ). ].
! !

!ConfigurationOfNBSQLite3 methodsFor: 'baselines' stamp: 'TorstenBergmann 9/27/2015 11:02'!
baseline13: spec
	<version: '1.3-baseline'>

	spec for: #'common' do: [
		spec blessing: #'baseline'.
		spec repository: 'http://smalltalkhub.com/mc/PharoExtras/NBSQLite3/main/'.
		spec 
			package: #'NBSQLite3-Core';
			package: #'NBSQLite3-Examples' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Glorp' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Tests-Core' with: [
				spec requires: #(#'NBSQLite3-Core' ). ];
			package: #'NBSQLite3-Tests-Glorp' with: [
				spec requires: #(#'NBSQLite3-Glorp' #'NBSQLite3-Tests-Core' ). ].
		spec 
			group: 'Core' with: #(#'NBSQLite3-Core' );
			group: 'default' with: #('Core' 'Tests');
			group: 'Examples' with: #(#'NBSQLite3-Examples' );
			group: 'Glorp' with: #(#'NBSQLite3-Glorp' );
			group: 'Tests' with: #(#'NBSQLite3-Tests-Core' );
			group: 'Tests-Glorp' with: #(#'NBSQLite3-Tests-Glorp' ). ].
! !

!ConfigurationOfNBSQLite3 methodsFor: 'symbolic versions' stamp: 'PierceNg 5/17/2015 18:33'!
bleedingEdge: spec
	<symbolicVersion: #'bleedingEdge'>

	spec for: #'common' version: '1.2'.
! !

!ConfigurationOfNBSQLite3 methodsFor: 'accessing' stamp: 'TorstenBergmann 12/15/2014 12:16'!
customProjectAttributes
	 "Edit to return a collection of any custom attributes e.g. for conditional loading: Array with: #'Condition1' with: #'Condition2.
	For more information see: http://code.google.com/p/metacello/wiki/CustomProjectAttrributes"

	^ #().! !

!ConfigurationOfNBSQLite3 methodsFor: 'symbolic versions' stamp: 'TorstenBergmann 9/27/2015 11:03'!
development: spec
	<symbolicVersion: #'development'>

	spec for: #'common' version: '1.3-baseline'.
! !

!ConfigurationOfNBSQLite3 methodsFor: 'accessing' stamp: 'TorstenBergmann 12/15/2014 12:16'!
project

	^ project ifNil: [ 
		"Bootstrap Metacello if it is not already loaded"
		(self class baseConfigurationClassIfAbsent: []) ensureMetacello.
		"Construct Metacello project"
		project := MetacelloMCProject new projectAttributes: self customProjectAttributes. 
		(Smalltalk at: #MetacelloVersionConstructor) on: self project: project.
		project loadType: #linear. "change to #atomic if desired"
		project ]! !

!ConfigurationOfNBSQLite3 methodsFor: 'symbolic versions' stamp: 'TorstenBergmann 9/27/2015 11:03'!
stable: spec
	<symbolicVersion: #'stable'>

	spec for: #'common' version: '1.2'.
! !

!ConfigurationOfNBSQLite3 methodsFor: 'versions' stamp: 'TorstenBergmann 1/17/2015 01:06'!
version10: spec
	<version: '1.0' imports: #('0.1-baseline' )>

	spec for: #'common' do: [
		spec blessing: #'stable'.
		spec description: 'version 1.0'.
		spec author: 'TorstenBergmann'.
		spec timestamp: '12/15/2014 12:22'.
		spec 
			package: #'NBSQLite3-Core' with: 'NBSQLite3-Core-TorstenBergmann.3';
			package: #'NBSQLite3-Examples' with: 'NBSQLite3-Examples-TorstenBergmann.1';
			package: #'NBSQLite3-Glorp' with: 'NBSQLite3-Glorp-TorstenBergmann.3';
			package: #'NBSQLite3-Tests-Core' with: 'NBSQLite3-Tests-Core-TorstenBergmann.2';
			package: #'NBSQLite3-Tests-Glorp' with: 'NBSQLite3-Tests-Glorp-TorstenBergmann.1'. ].
! !

!ConfigurationOfNBSQLite3 methodsFor: 'versions' stamp: 'PierceNg 4/9/2015 21:38'!
version11: spec
	<version: '1.1' imports: #('0.1-baseline' )>

	spec for: #'common' do: [
		spec blessing: #'stable'.
		spec description: 'version 1.1'.
		spec author: 'PierceNg'.
		spec timestamp: '4/9/2015 21:37'.
		spec 
			package: #'NBSQLite3-Core' with: 'NBSQLite3-Core-PierceNg.6';
			package: #'NBSQLite3-Examples' with: 'NBSQLite3-Examples-TorstenBergmann.1';
			package: #'NBSQLite3-Glorp' with: 'NBSQLite3-Glorp-TorstenBergmann.3';
			package: #'NBSQLite3-Tests-Core' with: 'NBSQLite3-Tests-Core-PierceNg.3';
			package: #'NBSQLite3-Tests-Glorp' with: 'NBSQLite3-Tests-Glorp-TorstenBergmann.1'. ].
! !

!ConfigurationOfNBSQLite3 methodsFor: 'versions' stamp: 'TorstenBergmann 9/27/2015 11:04'!
version12: spec
	<version: '1.2' imports: #('1.2-baseline' )>

	spec for: #'common' do: [
		spec blessing: #'stable'.
		spec description: 'version 1.2'.
		spec author: 'PierceNg'.
		spec timestamp: '5/16/2015 14:08'.
		spec 
			package: #'NBSQLite3-Core' with: 'NBSQLite3-Core-PierceNg.11';
			package: #'NBSQLite3-Tests-Core' with: 'NBSQLite3-Tests-Core-PierceNg.4';
			package: #'NBSQLite3-Examples' with: 'NBSQLite3-Examples-TorstenBergmann.1';
			package: #'NBSQLite3-Glorp' with: 'NBSQLite3-Glorp-TorstenBergmann.3';
			package: #'NBSQLite3-Tests-Glorp' with: 'NBSQLite3-Tests-Glorp-TorstenBergmann.1'. ].
! !

!ConfigurationOfNBSQLite3 methodsFor: 'versions' stamp: 'TorstenBergmann 9/27/2015 11:03'!
version13: spec
	<version: '1.3' imports: #('1.3-baseline' )>

	spec for: #'common' do: [
		spec blessing: #'bleedingEdge'.
		spec description: 'version 1.2'.
		spec author: 'PierceNg'.
		spec timestamp: '5/16/2015 14:08'.
		spec 
			package: #'NBSQLite3-Core' with: 'NBSQLite3-Core-PierceNg.11';
			package: #'NBSQLite3-Tests-Core' with: 'NBSQLite3-Tests-Core-PierceNg.4';
			package: #'NBSQLite3-Examples' with: 'NBSQLite3-Examples-TorstenBergmann.1';
			package: #'NBSQLite3-Glorp' with: 'NBSQLite3-Glorp-TorstenBergmann.3';
			package: #'NBSQLite3-Tests-Glorp' with: 'NBSQLite3-Tests-Glorp-TorstenBergmann.1'. ].
! !
